library couchclient;

import "dart:async";
import 'dart:io';
import 'dart:collection';
import 'dart:math' as math;
import 'dart:convert' show UTF8, JSON;
import 'package:logging/logging.dart';
import 'package:memcached_client/memcached_client.dart';

part 'src/CouchClient.dart';
part 'src/DesignDoc.dart';
part 'src/ObservedException.dart';
part 'src/ObservedModifiedException.dart';
part 'src/ObservedTimeoutException.dart';
part 'src/PersistTo.dart';
part 'src/Query.dart';
part 'src/ReplicateTo.dart';
part 'src/RestClient.dart';
part 'src/SpatialView.dart';
part 'src/SpatialViewDesign.dart';
part 'src/Stale.dart';
part 'src/View.dart';
part 'src/ViewDesign.dart';
part 'src/ViewResponse.dart';
part 'src/ViewRow.dart';
part 'src/ViewRowError.dart';

//config
part 'src/config/Bucket.dart';
part 'src/config/BucketMonitor.dart';
part 'src/config/BucketUpdateResponseHandler.dart';
part 'src/config/CacheConfig.dart';
part 'src/config/Config.dart';
part 'src/config/ConfigDifference.dart';
part 'src/config/ConfigFactory.dart';
part 'src/config/ConfigParserJson.dart';
part 'src/config/ConfigProvider.dart';
part 'src/config/ConfigType.dart';
part 'src/config/DefaultConfig.dart';
part 'src/config/Node.dart';
part 'src/config/Pool.dart';
part 'src/config/Port.dart';
part 'src/config/Reconfigurable.dart';
part 'src/config/ReconfigurableObserver.dart';
part 'src/config/Status.dart';
part 'src/config/Vbucket.dart';

//op/rest
part 'src/op/rest/AuthType.dart';
part 'src/op/rest/BucketType.dart';
part 'src/op/rest/CreateBucketOP.dart';
part 'src/op/rest/ListBucketsOP.dart';
part 'src/op/rest/ListDesignDocsOP.dart';

//op/views
part 'src/op/views/ViewBase.dart';
part 'src/op/views/DeleteDesignDocOP.dart';
part 'src/op/views/DeleteHttpOP.dart';
part 'src/op/views/DocsOP.dart';
part 'src/op/views/GetDesignDocOP.dart';
part 'src/op/views/GetHttpOP.dart';
part 'src/op/views/GetSpatialViewOP.dart';
part 'src/op/views/GetViewOP.dart';
part 'src/op/views/HttpOP.dart';
part 'src/op/views/HttpOPChannel.dart';
part 'src/op/views/NoDocsOP.dart';
part 'src/op/views/OnErrorType.dart';
part 'src/op/views/PostHttpOP.dart';
part 'src/op/views/PutDesignDocOP.dart';
part 'src/op/views/PutHttpOP.dart';
part 'src/op/views/ReducedOP.dart';
part 'src/op/views/SpatialViewRowNoDocs.dart';
part 'src/op/views/SpatialViewRowWithDocs.dart';
part 'src/op/views/ViewResponseNoDocs.dart';
part 'src/op/views/ViewResponseReduced.dart';
part 'src/op/views/ViewResponseWithDocs.dart';
part 'src/op/views/ViewRowNoDocs.dart';
part 'src/op/views/ViewRowReduced.dart';
part 'src/op/views/ViewRowWithDocs.dart';
part 'src/op/views/WithDocsOP.dart';

//spi
part 'src/spi/CouchClientImpl.dart';
part 'src/spi/CouchbaseConnection.dart';
part 'src/spi/CouchbaseConnectionFactory.dart';
part 'src/spi/CouchbaseMemcachedConnection.dart';
part 'src/spi/RestClientImpl.dart';
part 'src/spi/RestConnection.dart';
part 'src/spi/RestNode.dart';
part 'src/spi/VbucketNodeLocator.dart';
part 'src/spi/ViewConnection.dart';
part 'src/spi/ViewNode.dart';
